<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://fuji-9.com/
 * @since      1.0.0
 *
 * @package    Eus_For_Woocommerce
 * @subpackage Eus_For_Woocommerce/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    Eus_For_Woocommerce
 * @subpackage Eus_For_Woocommerce/public
 * @author     Fuji 9 <info@fuji-9.com>
 */
class Eus_For_Woocommerce_Public {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;

	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/eus-for-woocommerce-public.css', array(), $this->version, 'all' );

	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/eus-for-woocommerce-public.js', array( 'jquery' ), $this->version, false );

	}


	/**
     * pre_get_posts hook
     * 
     * @since    1.0.0
     * @param type $query
     */
    public function efw_pre_get_posts( $query = false ) {
        if(!is_admin() && is_search() ){
            $this->efw_disable_product_query($query);
        }
    }
    
    /**
     * woocommerce_product_query hook
     * 
     * @since    1.0.0
     * @param type $query
     */
    public function efw_woocommerce_product_query( $query = false ) {
    	if (function_exists('is_woocommerce')) {
	        if(!is_admin() && is_woocommerce() ){
	            $this->efw_disable_product_query($query);
	        }
	    }
    }

    /**
     * Related products query function
     * 
     * @since    1.0.0
     * @param array $query
     * @param type $this_id
     * @return type
     */
    public function efw_woocommerce_product_related_posts_query( $query, $this_id ) {

    	//Get excluded products
        $products_excluded = $this->get_excluded_products();
        if(empty($products_excluded)) {
            return $query;
        }

        $products_excluded = implode( ',', array_map( 'absint', $products_excluded ) );
        $query['where'] .= " AND p.ID NOT IN ( {$products_excluded} )";

        return $query;
    }

    /**
     * Related products array
     * 
     * @since    1.0.0
     * @param array $related_posts
     * @param int $product_id
     * @return type
     */
    public function efw_woocommerce_related_products($related_posts, $product_id) {
    	//Get excluded products
    	$products_excluded = $this->get_excluded_products();
    	foreach ($related_posts as $key => $related_post) {
    		$related_post = (int)$related_post;
    		if (in_array($related_post, $products_excluded)) {
    			unset($related_posts[$key]);
    		}
    	}
    	return $related_posts;
    }



    /**
     * Filter out products from shortcode
     * 
     * @since    1.0.0
     * @param array $related_posts
     * @param int $product_id
     * @return type
     */
	public function efw_woocommerce_shortcode_products_query( $args, $atts , $type) {

		//Get excluded products
	    $products_excluded = $this->get_excluded_products();

	    if (count($products_excluded) > 0) {
	    	//We need to check here if we are looking at single product shortcode or multiple one
	    	if ($type == 'product') {
	    		$product_id = (int)$atts['ids'];
	    		if (in_array($product_id, $products_excluded)) {
	    			$args['post_type'] = 'efw_non_exsistant_cpt';
	    		}
	    	} else {
	    		$args['post__not_in'] = $products_excluded;
	    	}
		}
		return $args;
	}

    /**
     * Used on single page for product
     * 
     * @since    1.0.0
     * @return redirect
     */
    public function efw_disable_single_product() {

    	if (function_exists('is_product')) {
	    	//If we are on product page
	        if ( is_product() ){

	        	//Get excluded products
	        	$products_excluded = $this->get_excluded_products();
	        	$product = get_queried_object();
	        	$product_id = $product->ID;

	        	//We redirect to home page, if user tries to access this product
	        	if (in_array($product_id, $products_excluded)) {
	        		wp_redirect( home_url());
	        	}
		    }
	    }
    }


    /**
     * Disable product query function
     * 
     * @since    1.0.0
     * @param type $query
     * @return type
     */
    private function efw_disable_product_query($query){
    	//Get excluded products
    	$products_excluded = $this->get_excluded_products();
    	if (count($products_excluded) > 0) {
    		$query->set( 'post__not_in', $products_excluded);
    	}
        return $query;
    }

    /**
     * Retrieve products that are not available for user
     * 
     * @since    1.0.0
     * @return array
     */
    private function get_excluded_products() {

    	//First check if user is admin, then return empty array since admins can view all products
  //   	if ( is_user_logged_in() ) {
		// 	if( current_user_can('administrator') ) {
		// 		return [];
		// 	}
		// }
		
		global $wpdb;

		//Get all A products that have meta_key eus_prerequisite_purchase
		$products_a_query = $wpdb->get_results( "SELECT post_id, meta_value FROM $wpdb->postmeta WHERE meta_key = 'eus_prerequisite_purchase' AND meta_value <> '-1' ");
		$excluded_ids = [];

		//For each of A products, check to see if user has purchased the B product
		foreach ( $products_a_query as $product ) {
			
			//Check subscribtion
			$has_sub = false;
			if (function_exists('wcs_user_has_subscription')) {
				$has_sub = wcs_user_has_subscription( '', $product->meta_value, 'active' );
			}

			//Check purchase
			if ( is_user_logged_in() ) {
				$current_user = wp_get_current_user();
				$has_purchased = false;
				if (function_exists('wc_customer_bought_product')) {
					$has_purchased = wc_customer_bought_product($current_user->user_email, $current_user->ID, $product->meta_value);
				}
			} else {
				//If user is not logged-in, we always return false
				$has_purchased = false;
			}

			//If user has not purchased or have active subscribtion to B product, add to excluded array product A
			if (!$has_sub or !$has_purchased) {
				array_push($excluded_ids, (int)$product->post_id);
			}
			
		}
		
		return $excluded_ids;
    }

}
